@@ -23,7 +23,6 @@ from django.conf import settings as dj_settings
 from django.core.signals import request_finished
 from django.db import models, transaction, IntegrityError
 from django.db.models import Sum
-from django.db.models.expressions import ExpressionNode
 from django.db.models.signals import post_save, post_delete, post_init, class_prepared
 from django.utils import timezone
 from django.utils.datastructures import SortedDict
@@ -36,9 +35,11 @@ from sentry.constants import STATUS_RESOLVED, STATUS_UNRESOLVED
 from sentry.processors.base import send_group_processors
 from sentry.signals import regression_signal
 from sentry.tasks.index import index_event
+from sentry.tasks.fetch_source import fetch_javascript_source
 from sentry.utils.cache import cache, Lock
 from sentry.utils.dates import get_sql_date_trunc
-from sentry.utils.db import get_db_engine, has_charts, resolve_expression_node
+from sentry.utils.db import get_db_engine, has_charts
+from sentry.utils.models import create_or_update
 from sentry.utils.queue import maybe_delay
 
 logger = logging.getLogger('sentry.errors')
@@ -95,7 +96,7 @@ class BaseManager(models.Manager):
             return self.model._meta.pk.name
         return key
 
-    def __make_key(self, prefix, kwargs):
+    def make_key(self, prefix, kwargs):
         kwargs_bits = []
         for k, v in sorted(kwargs.iteritems()):
             k = self.__prep_key(k)
@@ -176,7 +177,7 @@ class BaseManager(models.Manager):
         cache.delete(self.__get_lookup_cache_key(**{pk_name: instance.pk}))
 
     def __get_lookup_cache_key(self, **kwargs):
-        return self.__make_key('modelcache', kwargs)
+        return self.make_key('modelcache', kwargs)
 
     def contribute_to_class(self, model, name):
         super(BaseManager, self).contribute_to_class(model, name)
@@ -239,7 +240,7 @@ class BaseManager(models.Manager):
             return self.get(**kwargs), False
         except self.model.DoesNotExist:
             pass
-        lock_key = self.__make_key('lock', kwargs)
+        lock_key = self.make_key('lock', kwargs)
 
         # instance not found, lets grab a lock and attempt to create it
         with Lock(lock_key):
@@ -252,30 +253,7 @@ class BaseManager(models.Manager):
         return instance, created
 
     def create_or_update(self, **kwargs):
-        """
-        Similar to get_or_create, either updates a row or creates it.
-
-        The result will be (rows affected, False), if the row was not created,
-        or (instance, True) if the object is new.
-        """
-        defaults = kwargs.pop('defaults', {})
-
-        # before locking attempt to fetch the instance
-        affected = self.filter(**kwargs).update(**defaults)
-        if affected:
-            return affected, False
-        lock_key = self.__make_key('lock', kwargs)
-
-        # instance not found, lets grab a lock and attempt to create it
-        with Lock(lock_key) as lock:
-            if lock.was_locked:
-                affected = self.filter(**kwargs).update(**defaults)
-                return affected, False
-
-            for k, v in defaults.iteritems():
-                if isinstance(v, ExpressionNode):
-                    kwargs[k] = resolve_expression_node(self.model(), v)
-            return self.create(**kwargs), True
+        return create_or_update(self.model, **kwargs)
 
 
 class ScoreClause(object):
@@ -525,6 +503,13 @@ class GroupManager(BaseManager, ChartMixin):
                 transaction.rollback_unless_managed(using=group._state.db)
                 logger.exception(u'Error indexing document: %s', e)
 
+        if settings.SCRAPE_JAVASCRIPT_CONTEXT and event.platform == 'javascript' and not is_sample:
+            try:
+                maybe_delay(fetch_javascript_source, event)
+            except Exception, e:
+                transaction.rollback_unless_managed(using=group._state.db)
+                logger.exception(u'Error fetching javascript source: %s', e)
+
         if is_new:
             try:
                 regression_signal.send_robust(sender=self.model, instance=group)
@@ -584,12 +569,15 @@ class GroupManager(BaseManager, ChartMixin):
                 extra['message'] = message
 
             if group.status == STATUS_RESOLVED:
-                # Group has changed from resolved -> unresolved
-                is_new = True
+                # Makin things atomic
+                is_new = bool(self.filter(
+                    id=group.id,
+                ).exclude(
+                    status=STATUS_UNRESOLVED,
+                ).update(active_at=date, status=STATUS_UNRESOLVED))
 
-                # We have to perform this update inline, as waiting on buffers means
-                # this event could be treated as "new" several times
-                group.update(active_at=date, status=STATUS_UNRESOLVED)
+                group.active_at = date
+                group.status = STATUS_UNRESOLVED
 
             group.last_seen = extra['last_seen']
 
@@ -646,7 +634,7 @@ class GroupManager(BaseManager, ChartMixin):
 
         user_ident = event.user_ident
         if user_ident:
-            self.record_affected_user(group, user_ident)
+            self.record_affected_user(group, user_ident, event.data.get('sentry.interfaces.User'))
 
         try:
             self.add_tags(group, tags)
@@ -655,18 +643,44 @@ class GroupManager(BaseManager, ChartMixin):
 
         return group, is_new, is_sample
 
-    def record_affected_user(self, group, user_ident):
-        from sentry.models import AffectedUserByGroup
+    def record_affected_user(self, group, user_ident, data=None):
+        from sentry.models import TrackedUser, AffectedUserByGroup
 
         project = group.project
         date = group.last_seen
 
+        if data:
+            email = data.get('email')
+        else:
+            email = None
+
+        # TODO: we should be able to chain the affected user update so that tracked
+        # user gets updated serially
+        tuser = TrackedUser.objects.get_or_create(
+            project=project,
+            ident=user_ident,
+            defaults={
+                'email': email,
+                'data': data,
+            }
+        )[0]
+
+        app.buffer.incr(TrackedUser, {
+            'num_events': 1,
+        }, {
+            'id': tuser.id,
+        }, {
+            'last_seen': date,
+            'email': email,
+            'data': data,
+        })
+
         app.buffer.incr(AffectedUserByGroup, {
             'times_seen': 1,
         }, {
             'group': group,
             'project': project,
-            'ident': user_ident,
+            'tuser': tuser,
         }, {
             'last_seen': date,
         })
