@@ -43,7 +43,7 @@ class NewProjectForm(forms.ModelForm):
         widget=forms.TextInput(attrs={'placeholder': _('example.com')}))
 
     class Meta:
-        fields = ('name',)
+        fields = ('name', 'platform')
         model = Project
 
 
@@ -51,7 +51,7 @@ class NewProjectAdminForm(NewProjectForm):
     owner = UserField(required=False)
 
     class Meta:
-        fields = ('name', 'owner')
+        fields = ('name', 'platform', 'owner')
         model = Project
 
 
@@ -99,12 +99,14 @@ class RemoveProjectForm(forms.Form):
 
 
 class EditProjectForm(forms.ModelForm):
-    public = forms.BooleanField(required=False, help_text=_('Allow anyone (even anonymous users) to view this project'))
+    public = forms.BooleanField(required=False,
+        help_text=_('Allow anyone (even anonymous users) to view this project'))
     team = forms.TypedChoiceField(choices=(), coerce=int)
-    origins = OriginsField(required=False)
+    origins = OriginsField(label=_('Allowed Domains'), required=False,
+        help_text=_('Separate multiple entries with a newline.'))
 
     class Meta:
-        fields = ('name', 'public', 'team')
+        fields = ('name', 'platform', 'public', 'team')
         model = Project
 
     def __init__(self, request, team_list, data, instance, *args, **kwargs):
@@ -121,10 +123,15 @@ class EditProjectForm(forms.ModelForm):
 
     def clean_team(self):
         value = self.cleaned_data.get('team')
-        if not value or value is -1:
+        if not value:
             return
 
-        if value == self.instance.team.id:
+        # TODO: why is this not already an int?
+        value = int(value)
+        if value == -1:
+            return
+
+        if self.instance.team and value == self.instance.team.id:
             return self.instance.team
 
         return self.team_list[value]
@@ -135,5 +142,5 @@ class EditProjectAdminForm(EditProjectForm):
     owner = UserField(required=False)
 
     class Meta:
-        fields = ('name', 'public', 'team', 'owner')
+        fields = ('name', 'platform', 'public', 'team', 'owner')
         model = Project
