@@ -21,7 +21,7 @@ from sentry.constants import (SORT_OPTIONS, SEARCH_SORT_OPTIONS,
     SORT_CLAUSES, MYSQL_SORT_CLAUSES, SQLITE_SORT_CLAUSES, MEMBER_USER,
     SCORE_CLAUSES, MYSQL_SCORE_CLAUSES, SQLITE_SCORE_CLAUSES)
 from sentry.filters import get_filters
-from sentry.models import Group, Event, SearchDocument
+from sentry.models import Group, Event, SearchDocument, Activity
 from sentry.permissions import can_admin_group
 from sentry.plugins import plugins
 from sentry.utils import json
@@ -117,7 +117,12 @@ def _get_group_list(request, project):
     if score_clause:
         event_list = event_list.extra(
             select={'sort_value': score_clause},
-        ).order_by('-sort_value', '-last_seen')
+        )
+        # HACK: dont sort by the same column twice
+        if sort == 'date':
+            event_list = event_list.order_by('-last_seen')
+        else:
+            event_list = event_list.order_by('-sort_value', '-last_seen')
         cursor = request.GET.get('cursor')
         if cursor:
             event_list = event_list.extra(
@@ -250,9 +255,13 @@ def render_with_group_context(group, template, context, request=None):
 
 @has_group_access
 def group(request, project, group):
+    activity = Activity.objects.filter(
+        group=group,
+    ).order_by('-datetime').select_related('user')
 
     return render_with_group_context(group, 'sentry/groups/details.html', {
         'page': 'details',
+        'activity': activity,
     }, request)
 
 
@@ -298,7 +307,7 @@ def group_event_list(request, project, group):
 
 @has_access(MEMBER_USER)
 def group_event_list_json(request, project, group_id):
-    group = get_object_or_404(Group, pk=group_id, project=project)
+    group = get_object_or_404(Group, id=group_id, project=project)
 
     limit = request.GET.get('limit', settings.MAX_JSON_RESULTS)
     try:
@@ -315,13 +324,26 @@ def group_event_list_json(request, project, group_id):
 
 @has_group_access
 def group_event_details(request, project, group, event_id):
-    event = get_object_or_404(group.event_set, pk=event_id)
+    event = get_object_or_404(group.event_set, id=event_id)
+
+    base_qs = group.event_set.exclude(id=event_id)
+    try:
+        next_event = base_qs.filter(datetime__gte=event.datetime).order_by('datetime')[0:1].get()
+    except Event.DoesNotExist:
+        next_event = None
+
+    try:
+        prev_event = base_qs.filter(datetime__lte=event.datetime).order_by('-datetime')[0:1].get()
+    except Event.DoesNotExist:
+        prev_event = None
 
     return render_to_response('sentry/groups/event.html', {
         'project': project,
         'page': 'event',
         'group': group,
         'event': event,
+        'next_event': next_event,
+        'prev_event': prev_event,
         'json_data': event.data.get('extra', {}),
         'version_data': event.data.get('modules', None),
         'can_admin_event': can_admin_group(request.user, group),
